package com.suliang.common.base.activity

import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import android.widget.Toast
import androidx.annotation.StringRes
import androidx.viewbinding.ViewBinding
import com.suliang.common.base.LoadingDialog
import com.suliang.common.extension.initBinding
import com.suliang.common.util.ActivityStackManager
import com.zackratos.ultimatebarx.ultimatebarx.statusBarOnly
import com.suliang.common.base.ViewBehavior
import com.suliang.common.util.LogUtil


/**
 * 基类Activity
 */
abstract class BaseActivity<VB : ViewBinding> : LifecycleLogActivity(), ViewBehavior {

    private var _binding: VB? = null

    val binding: VB
        get() = _binding!!

//    val binding : VB by lazy {
//        initViewBinding()
//    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        //入栈
        ActivityStackManager.addActivity(this)
        initFirst()
        initActivity(savedInstanceState)
        loadData()
    }

    /***
     * 实例化binding，
     */
    protected open fun initFirst() {
        setLayoutBefore()
        _binding = initBinding(layoutInflater)
        setContentView(binding.root)
        initStatusBar()
        setLayoutAfter()
    }

    /**
     * 状态栏初始化方法
     */
    open fun initStatusBar() {
        statusBarOnly {
            fitWindow = true
            color = Color.WHITE
            light = true
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        ActivityStackManager.removeActivity(this)
        _binding = null
    }

    /** 布局前操作 ： 空实现 */
    open fun setLayoutBefore() {

    }

    /**
     * 布局后操作：空实现
     */
    open fun setLayoutAfter() {

    }

    /**
     * onCreate中给与app中用户的具体实现
     * binding 与 ViewModel都已实现好
     * @param savedInstanceState Bundle?
     */
    abstract fun initActivity(savedInstanceState: Bundle?)

    /** onCreate()最后实现的数据加载调用 */
    abstract fun loadData()

    private val loadingDialog by lazy {
        LoadingDialog()
    }

    /** toast提示 */
    override fun showToast(event: ToastEvent) {
        val showTime = if (event.showLong) Toast.LENGTH_LONG else Toast.LENGTH_LONG
        event.content?.let {
            Toast.makeText(this, it, showTime).show()
        } ?: let {
            Toast.makeText(this, event.contentResId!!, showTime).show()
        }
    }

    protected fun showToast(text: String, showLong: Boolean = false) {
        showToast(ToastEvent(content = text, showLong = showLong))
    }

    protected fun showToast(@StringRes resId: Int, showLong: Boolean = false) {
        showToast(ToastEvent(contentResId = resId, showLong = showLong))
    }

    override fun showHideLoading(isShow: Boolean) {
        if (isShow) {
            loadingDialog.show(supportFragmentManager, javaClass.name)
        } else if (loadingDialog.activity != null){
            loadingDialog.dismiss()
        }
    }

    override fun startActivity(clazz: Class<*>) {
        startActivity(Intent(this,clazz))
    }


    // 反射实现 ViewBinding
//    @SuppressWarnings("unchecked")
//    fun initBinding(inflater: LayoutInflater): VB {
//        val vbClass =
//            (javaClass.genericSuperclass as ParameterizedType).actualTypeArguments.filterIsInstance<Class<VB>>()
//        val inflate = vbClass[0].getDeclaredMethod("inflate", LayoutInflater::class.java)
//        return inflate.invoke(null, inflater) as VB
//    }

//    https://juejin.cn/post/6926007534188773384#heading-1
//    private fun initViewBinding(): VB{
//        return reflexViewBinding(this::class.java,layoutInflater)
//    }
//    private  fun <V : ViewBinding?> reflexViewBinding(aClass: Class<*>, from: LayoutInflater?): V {
//        return try {
//            val genericSuperclass: Type = aClass.genericSuperclass
//            if (genericSuperclass is ParameterizedType) {
//                val actualTypeArguments: Array<Type> =
//                    (Objects.requireNonNull(genericSuperclass) as ParameterizedType).getActualTypeArguments()
//                for (actualTypeArgument in actualTypeArguments) {
//                    var tClass: Class<*>
//                    tClass = try {
//                        actualTypeArgument as Class<*>
//                    } catch (e: Exception) {
//                        continue
//                    }
//                    if (ViewBinding::class.java.isAssignableFrom(tClass)) {
//                        val inflate: Method = tClass.getMethod("inflate", LayoutInflater::class.java)
//                        return inflate.invoke(null, from) as V
//                    }
//                }
//            }
//            reflexViewBinding(aClass.superclass, from)
//        } catch (e: Exception) {
//            e.printStackTrace()
//            throw RuntimeException(e.message, e)
//        }
//    }


}