package com.suliang.common.base.adapter

import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView

abstract class BaseAdapter<T> :
    RecyclerView.Adapter<BaseAdapterViewHolder>() {

    companion object{
        val TYPE_EMPTY = -1
    }

    //数据源
    private var mData: MutableList<T> = mutableListOf<T>()

    /** 是否需要显示空布局 */
    var needShowEmptyView = false

    /**
     * 当前位置是否需要显示空布局，主要是为0的时候，这个判断才会有效
     * @param position Int
     */
    private fun enableEmptyPosition(position:Int): Boolean{
        return position == 0 && mData.isEmpty() && needShowEmptyView
    }

    override fun getItemCount(): Int {
        return if (needShowEmptyView && mData.isEmpty()) 1 else mData.size
    }

    override fun getItemViewType(position: Int): Int {
        if (enableEmptyPosition(position)){
            return TYPE_EMPTY
        }
        return super.getItemViewType(position)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseAdapterViewHolder {
        return when(viewType){
            TYPE_EMPTY -> coverEmptyViewHolder(parent)
            else -> coverViewHolder(parent,viewType)
        }
    }

    override fun onBindViewHolder(holder: BaseAdapterViewHolder, position: Int) {
        if (enableEmptyPosition(position)){
            onBindEmptyViewHolder(holder)
        }else{
            onBindVH(holder,position)
        }
    }

    /** 创建空布局的ViewHolder 如果needShowEmpty = false 则可以不创建 */
    abstract fun coverEmptyViewHolder(parent: ViewGroup): BaseAdapterViewHolder
    /** 空布局的数据绑定 */
    abstract fun onBindEmptyViewHolder(holder: BaseAdapterViewHolder)
    /** 创建 正常item的 ViewHolder */
    abstract fun coverViewHolder(parent: ViewGroup,viewType: Int): BaseAdapterViewHolder
    /** 绑定 正常item 的数据 */
    abstract fun onBindVH(holder: BaseAdapterViewHolder,position: Int)



    /**
     * 设置数据,并更新列表
     * @param data List<T>?
     */
    fun setData(data: MutableList<T>?) {
        data?.let { it ->       //不为空
            mData = it
        } ?: let {      //为空
            mData = mutableListOf()
        }
        notifyItemRangeChanged(0, mData.size)
    }









    /**
     * 添加单条数据
     * @param data T?
     * @param position Int?
     */
    fun addData(data: T?, position: Int?) {
        data?.let {
            val size = mData.size
            val startPosition = position?.let { it ->
                when {
                    it < 0 -> 0
                    it >= size -> size
                    else -> it
                }
            } ?: size
            mData.add(startPosition,data)
            notifyItemInserted(startPosition)
            notifyItemRangeChanged(startPosition,mData.size + 1)
        }
    }

    /**
     * 添加数据集
     * @param data List<T>? 数据集
     * @param position Int? 添加到指定位置，可空
     */
    fun addData(data: List<T>?, position: Int? = null) {
        if (!data.isNullOrEmpty()) {
            val size = mData.size
            val startPosition = position?.let { it ->
                when {
                    it < 0 -> 0
                    it >= size -> size
                    else -> it
                }
            } ?: size
            mData.addAll(startPosition, data)
            //通知有新的item插入进来了
            notifyItemInserted(startPosition)
            //范围性更新，直接调用onBindViewHolder
            notifyItemRangeChanged(startPosition, mData.size + 1 )
        }
    }

}