package com.suliang.common.base.fragment

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.annotation.StringRes
import androidx.databinding.ViewDataBinding
import androidx.fragment.app.Fragment
import androidx.viewbinding.ViewBinding
import com.suliang.common.base.LoadingDialog
import com.suliang.common.base.ViewBehavior
import com.suliang.common.base.activity.ToastEvent
import com.suliang.common.extension.initBinding

/**
 * Fragment ViewBinding基类，封装ViewDataBinding  自带懒加载
 *
 * TabLayout+ViewPager懒加载： 关联使用 tabLayout.setupWithViewPager(ViewPager),
 * 适配器使用两参构造，第二参数使用：BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT，否则会走setUserVisibleHint
 *
 * FragmentStatePagerAdapter会调用onSaveInstanceState方法来保存Fragment的状态
 *
 * 通过add + show + hide 模式加载Fragment
 *
 * @property VB : ViewDataBinding
 */
abstract class BaseFragment<VB : ViewBinding> : Fragment(),ViewBehavior {

    private var _binding: VB? = null

    val binding: VB
        get() = _binding!!

    //是否加载数据
    private var isLoaded = false

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View? {
        _binding = initBinding(inflater, container)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initFirst()
        initFragment()
    }

    override fun onResume() {
        super.onResume()
        if (!isLoaded && !isHidden) {
            loadData()
            isLoaded = true
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        _binding = null
    }

    /**
     * onViewCreated中进行的初始：多数时候可以是空实现，主要在使用VM时进行vm的初始
     */
    abstract fun initFirst()

    /**
     * onViewCreated() 后进行的Fragment初始设置
     */
    abstract fun initFragment()

    /**
     * 界面显示出来后再加载数据
     */
    abstract fun loadData()



    private val loadingDialog by lazy {
        LoadingDialog()
    }

    /** toast提示 */
    override fun showToast(event: ToastEvent) {
        val showTime = if (event.showLong) Toast.LENGTH_LONG else Toast.LENGTH_LONG
        event.content?.let {
            Toast.makeText(requireContext(), it, showTime).show()
        } ?: let {
            Toast.makeText(requireContext(), event.contentResId!!, showTime).show()
        }
    }

    protected fun showToast(text: String, showLong: Boolean = false) {
        showToast(ToastEvent(content = text, showLong = showLong))
    }

    protected fun showToast(@StringRes resId: Int, showLong: Boolean = false) {
        showToast(ToastEvent(contentResId = resId, showLong = showLong))
    }

    override fun showHideLoading(isShow: Boolean) {
        if (isShow) {
            loadingDialog.show(childFragmentManager, javaClass.name)
        } else if (loadingDialog.activity != null){
            loadingDialog.dismiss()
        }
    }

    override fun startActivity(clazz: Class<*>) {
        startActivity(Intent(requireActivity(),clazz))
    }

}