package com.suliang.common.extension

import android.content.Intent
import androidx.annotation.IdRes
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentActivity
import androidx.fragment.app.FragmentManager
import androidx.lifecycle.Lifecycle

/**
 * author suliang
 * create 2022/3/7 14:30
 * Describe:  扩展：  fragment的显示 隐藏
 */

/*----------------------------------Activity  Start-------------------------------------*/

fun <T> FragmentActivity.startActivity(newActivityClass:Class<T>){
    this.startActivity(Intent(this,newActivityClass))
}



/**
 * Activity 级别 加载Fragment集合进入容器，并显示
 * @receiver FragmentActivity
 * @param containerId Int
 * @param showPosition Int
 * @param fragment Array<out Fragment>
 */
fun FragmentActivity.loadFragment(@IdRes containerId: Int,showPosition: Int = 0,vararg fragment: Fragment){
    loadFragmentTransaction(containerId,showPosition,supportFragmentManager,*fragment)
}

/**
 * Activity 显示指定Fragment并隐藏其他Fragment
 * @receiver FragmentActivity
 * @param showFragment Fragment
 */
fun FragmentActivity.showHideFragment(showFragment: Fragment){
    showHideFragmentTransaction(supportFragmentManager,showFragment)
}


/*----------------------------------Activity  End-------------------------------------*/


/*----------------------------------Fragment  Start-------------------------------------*/

/**
 * Fragment 添加 子Fragment进入容器,并显示[showPosition]位置的fragment
 * @param containerId Int
 * @param showPosition Int
 * @param fragments Array<out Fragment>
 */
fun Fragment.loadFragment(@IdRes containerId: Int,showPosition: Int,vararg fragments: Fragment){
    loadFragmentTransaction(containerId,showPosition,childFragmentManager,*fragments)
}

/**
 * Fragment 显示指定的子Fragment
 * @receiver Fragment
 * @param showFragment Fragment
 */
fun Fragment.showHideFragment(showFragment: Fragment){
    showHideFragmentTransaction(childFragmentManager,showFragment)
}


/*----------------------------------Fragment  End-------------------------------------*/


/**
 * 使用 add + show + hide 实现懒加载
 * 默认显示[showPosition]位置的fragment，最大[Lifecycle]为[Lifecycle.State.RESUMED]
 * @param containerId Int  容器
 * @param showPosition Int  显示位置
 * @param supportFragmentManager FragmentManager
 * @param fragment Array<out Fragment>  集合
 */
private fun loadFragmentTransaction(
    @IdRes containerId: Int,
    showPosition: Int,
    fragmentManager: FragmentManager,
    vararg fragment : Fragment
) {
    when(fragment.isNotEmpty()){
        true -> {
            fragmentManager.beginTransaction().apply {
                fragment.forEachIndexed{ index, fragment ->
                    //添加fragment，并设置有tag
                    add(containerId,fragment,fragment::javaClass.name)
                    //设置什么周期
                    when(index){
                        showPosition -> {
                            setMaxLifecycle(fragment,Lifecycle.State.RESUMED)
                        }
                        else -> {
                            hide(fragment)
                            setMaxLifecycle(fragment,Lifecycle.State.STARTED)
                        }
                    }
                }
            }.commit()
        }
        else -> throw  IllegalStateException("fragments must not empty!")
    }
}

/**
 * 显示指定Fragment且隐藏其他Fragment
 * @param fragmentManager FragmentManager
 * @param showFragment Fragment
 */
private fun showHideFragmentTransaction(fragmentManager: FragmentManager, showFragment: Fragment) {
    fragmentManager.beginTransaction().apply {
        fragmentManager.fragments.forEach{
            when(it){
                showFragment -> {
                    show(it)
                    setMaxLifecycle(it,Lifecycle.State.RESUMED)
                }
                else -> {
                    hide(it)
                    setMaxLifecycle(it,Lifecycle.State.STARTED)
                }
            }
        }
    }.commit()
}