package com.suliang.common.util.image

import android.graphics.Bitmap
import android.graphics.drawable.Drawable
import com.bumptech.glide.Glide
import com.bumptech.glide.RequestBuilder
import com.bumptech.glide.load.MultiTransformation
import com.bumptech.glide.load.Transformation
import com.bumptech.glide.load.resource.bitmap.*
import com.bumptech.glide.load.resource.gif.GifDrawable
import com.suliang.common.util.AppGlobals
import jp.wasabeef.glide.transformations.BlurTransformation
import java.io.File
import kotlin.concurrent.thread


/**
 * author suliang
 * create 2022/3/11 16:14
 * Describe:  Glide 加载的具体实现
 * https://juejin.cn/post/6844903463134953479#heading-3
 */
internal class GlideLoaderStrategy : IBaseLoaderStrategy {

    override fun clearMemoryCache() {
        Glide.get(AppGlobals.application).clearMemory()
    }

    override fun clearDiskCache() {
        thread {
            Glide.get(AppGlobals.application).clearDiskCache()
        }
    }

    override fun loadImage(option: ImageLoaderOption) {
        //如果上下文和图片目标有一个不存在，则报异常
        if (option.context == null && option.targetView == null) {
            throw Exception("the Glide Option context or targetView is not empty")
        }
        if (option.context == null) {
            option.context = option.targetView!!.context
        }
        option.context?.let {
            load(option)
        }
    }


    private fun initNormalDrawableGlideRequest(option: ImageLoaderOption): RequestBuilder<Drawable> {
        return GlideApp.with(option.context!!).asDrawable().load(source(option)).placeholder(option.placeholderResId)
                .error(option.errorResId).fallback(option.fallback)
    }

    private fun initNormalFileGlideRequest(option: ImageLoaderOption): RequestBuilder<File> {
        return GlideApp.with(option.context!!).asFile().load(source(option)).placeholder(option.placeholderResId)
                .error(option.errorResId).fallback(option.fallback)
    }

    private fun initNormalBitmapGlideRequest(option: ImageLoaderOption): RequestBuilder<Bitmap> {
        return GlideApp.with(option.context!!).asBitmap().load(source(option)).placeholder(option.placeholderResId)
                .error(option.errorResId).fallback(option.fallback)
    }

    private fun initNormalGifGlideRequest(option: ImageLoaderOption): RequestBuilder<GifDrawable> {
        return GlideApp.with(option.context!!).asGif().load(source(option)).placeholder(option.placeholderResId)
                .error(option.errorResId).fallback(option.fallback)
    }

    /** 普通加载 */
    private fun load(option: ImageLoaderOption) {
        when {
            option.asBitmap -> {
                initNormalBitmapGlideRequest(option).transform(initTranslation(option)).into(option.targetView!!)
            }
            option.asGif -> {
                initNormalGifGlideRequest(option).transform(initTranslation(option)).into(option.targetView!!)
            }
            option.asFile -> {
                initNormalFileGlideRequest(option).transform(initTranslation(option)).into(option.targetView!!)
            }
            else -> {
                initNormalDrawableGlideRequest(option).transform(initTranslation(option)).into(option.targetView!!)
            }
        }
    }


    private fun initTranslation(option: ImageLoaderOption): MultiTransformation<Bitmap> {
        val list: MutableList<Transformation<Bitmap>> = ArrayList()
        list.add(BlurTransformation(25, 3))
        //gravity 模式
        if (option.centerInsideEnable) {
            list.add(CenterInside())
        } else if (option.fitCenterEnable) {
            list.add(FitCenter())
        } else {
            list.add(CenterCrop())
        }
        //圆形
        if (option.circleEnable) {
            list.add(CircleCrop())
        }
        //圆角
        if (option.rounderCorners != 0) {
            list.add(RoundedCorners(option.rounderCorners))
        }
        //模糊
        if (option.blurEanble) {
            list.add(BlurTransformation(25, 3))
        }
        return MultiTransformation(list)
    }

    /** 最终加载资源*/
    private fun source(option: ImageLoaderOption) =
        option.bitmap ?: option.drawable ?: option.uri ?: option.url ?: option.file ?: option.byteArray
        ?: option.resourceId


//    /** 加载圆角图片 */
//    private fun loadRounderCorner(option: ImageLoaderOption) {
//        when {
//            option.asBitmap -> {
//                initNormalBitmapGlideRequest(option).transform(RoundedCorners(option.rounderCorners))
//                        .into(option.targetView!!)
//            }
//            option.asGif -> {
//                initNormalGifGlideRequest(option).transform(RoundedCorners(option.rounderCorners))
//                        .into(option.targetView!!)
//            }
//            option.asFile -> {
//                initNormalFileGlideRequest(option).transform(RoundedCorners(option.rounderCorners))
//                        .into(option.targetView!!)
//            }
//            else -> {
//                initNormalDrawableGlideRequest(option).transform(RoundedCorners(option.rounderCorners))
//                        .into(option.targetView!!)
//            }
//        }
////        GlideApp.with(option.context).load(source(option)).transform(RoundedCorners(option.rounderCorners))
////                .into(option.targetView!!)
//    }
//
//    /** 加载圆形图片 */
//    private fun loadCircle(option: ImageLoaderOption) {
//        when {
//            option.asBitmap -> {
//                initNormalBitmapGlideRequest(option).transform(CircleCrop()).into(option.targetView!!)
//            }
//            option.asGif -> {
//                initNormalGifGlideRequest(option).transform(CircleCrop()).into(option.targetView!!)
//            }
//            option.asFile -> {
//                initNormalFileGlideRequest(option).transform(CircleCrop()).into(option.targetView!!)
//            }
//            else -> {
//                initNormalDrawableGlideRequest(option).transform(CircleCrop()).into(option.targetView!!)
//            }
//        }
//        GlideApp.with(option.context).load(source(option)).transform(CircleCrop()).into(option.targetView!!)
//    }
//
//    /** 高斯模糊 */
//    private fun loadBlur(option: ImageLoaderOption) {
//        val list: MutableList<Transformation<Bitmap>> = ArrayList()
//        list.add(BlurTransformation(25, 3))
//
//        when {
//            option.centerInsideEnable -> list.add(CenterInside())
//            option.fitCenterEnable -> list.add(FitCenter())
//            option.circleEnable -> list.add(CircleCrop())       //圆形
//        }
//
//        list.add(CenterCrop())
//
//
//        val multiTransformation = MultiTransformation(list)
//        GlideApp.with(option.context).load(source(option)).transform(multiTransformation).into(option.targetView!!)
//    }


}