package com.suliang.common.util.media

import android.media.MediaPlayer
import android.os.Handler
import android.os.Looper
import com.suliang.common.util.LogUtil
import kotlin.concurrent.thread

/**
 * author suliang
 * create 2022/3/8 17:04
 * Describe:  MediaPlayer播放器
 *  正常使用：先调用play  退出时需释放
 *          seekTo 播放时，可直接使用， 暂停时，需在回调后调用start
 *  internal 模块内可见
 *
 */
internal class MPUtil : IMP,MediaPlayer.OnPreparedListener,MediaPlayer.OnErrorListener,MediaPlayer.OnCompletionListener,MediaPlayer.OnInfoListener,MediaPlayer.OnSeekCompleteListener{


    private var mediaPlayer: MediaPlayer? = null   //播放器
    private var urlPath: String? = null        //播放路径
    private var listener: IMPListener? = null         //播放监听器
    private var currentState: EMediaState = EMediaState.ERROR      //播放状态
    private var mTargetPosition = -1     //拖动位置

    private fun MediaPlayer.resetPlayer() {
        setOnPreparedListener(null)
        setOnErrorListener(null)
        setOnCompletionListener(null)
        setOnInfoListener(null)
        setOnSeekCompleteListener(null)
        mTargetPosition = -1
        try {
            reset()
        } catch (e: Exception) {
            LogUtil.e("MediaPlayer reset释放异常  $currentState")
            e.printStackTrace()
        }
    }

    override fun play(url: String, position: Int,listener: IMPListener?) {
        listener?.let {
            this.listener = it
        }
        if (url.isEmpty()) {
            handleError("url路径为空")
            return
        }
        //初始化MediaPlayer
        initMedia()
        urlPath = url
        //拖动到开始进行播放
        seekTo(0)
    }

    /**
     * MediaPlayer初始化，设置监听
     */
    private fun initMedia() {
        mediaPlayer?.resetPlayer() ?: let {
            it.mediaPlayer = MediaPlayer()
        }
        mediaPlayer?.apply {
            setOnPreparedListener(mOnPreparedLister)
            setOnErrorListener(this@MPUtil)
            setOnCompletionListener(this@MPUtil)
            setOnInfoListener(this@MPUtil)
            setOnSeekCompleteListener(this@MPUtil)
        }
        currentState = EMediaState.IDLE
        handleListener()
        urlPath = null
    }



    /** mediaPlayer的seekTo 完成后有监听回调，在播放中会跳到指定地方，暂停时跳到指定位置后需手动调用播放
     * 1、initMedia后调用该方法
     * 2、在外部直接调用该方法(必须是播放或暂停状态才有效果)
     * */
    override fun seekTo(targetPosition: Int) {
        //路径错误，返回报错
        if (urlPath.isNullOrEmpty()) {
            handleError("seek url路径为空")
            return
        }
        //修改定位标志
        mTargetPosition = if (targetPosition < 0) 0 else targetPosition

        //根据状态开始操作
        if (currentState == EMediaState.IDLE) {     //空闲状态，开始设置资源
            try {
                mediaPlayer?.setDataSource(urlPath)     //设置资源
                currentState = EMediaState.INITIALIZED  //idle -> init
                handleListener()
            } catch (e: Exception) {
                handleError("MediaPlayer设置资源失败")
                e.printStackTrace()
                return
            }
        }
        when (currentState) {
            EMediaState.INITIALIZED -> {     // 设置好资源了，开始播放准备
                currentState = EMediaState.PREPARING
                handleListener()
                mediaPlayer?.prepareAsync()
            }
            EMediaState.PREPARED -> {         //准备好了,进行操作
                handlePrepared()
            }
            EMediaState.RUNNING, EMediaState.PAUSE -> {      //播放或暂停时，直接跳到指定位置
                mediaPlayer?.seekTo(mTargetPosition)        // 拖动播放（播放或暂停时重新检查状态处理）
            }
            else -> {
                handleError("seekTo时状态异常" )
            }
        }
    }

    private val mOnPreparedLister: MediaPlayer.OnPreparedListener = MediaPlayer.OnPreparedListener {

    }

    /** 异步准备完毕 ,开始播放*/
    private fun handlePrepared() {
        if (currentState == EMediaState.PREPARED) {         //修改为播放状态，调用播放
            LogUtil.i("准备完成，开始播放")
            mediaPlayer?.start()
            currentState = EMediaState.RUNNING
            handleListener()
            checkSeekPlay()
        } else {
            handleError("状态异常，这里只能是 PREPARED 而现在是：$currentState")
        }
    }

//    private val mOnSeekCompleteListener: MediaPlayer.OnSeekCompleteListener = MediaPlayer.OnSeekCompleteListener {
//        LogUtil.i("mOnSeekCompleteListener 回调")
//        handleSeekComplete()
//    }
//
//
//    private val mOnCompleteListener: MediaPlayer.OnCompletionListener = MediaPlayer.OnCompletionListener {
//        handleComplete()
//    }
//
//
//    private val mOnErrorListener: MediaPlayer.OnErrorListener = MediaPlayer.OnErrorListener { _, what, extra ->
//        handleError("mOnErrorListener 异常调用 $extra")
//        true
//    }
//
//    private val mOnInfoListener: MediaPlayer.OnInfoListener = MediaPlayer.OnInfoListener { mp, what, extra ->
//        false
//    }

    private fun handleSeekComplete() {
        when (currentState) {
            EMediaState.PREPARED -> handlePrepared()
            EMediaState.RUNNING,EMediaState.PAUSE -> checkSeekPlay()
            else -> {
                handleError("seekComplete state error : ${currentState.name}")
            }
        }
    }

    private fun checkSeekPlay() {
        mediaPlayer?.let {
            //拖动只有播放中才有效
            if (currentState == EMediaState.RUNNING || currentState == EMediaState.PAUSE) {
                if (mTargetPosition >= 0) {      //如果拖动大于等于0
                    it.seekTo(mTargetPosition)
                    mTargetPosition = -1
                }
                if (!it.isPlaying) { //如果暂停，则开始播放
                    it.start()
                }
            } else {
                handleError("checkSeekPlay 非播放状态")
            }
        }
    }

    private fun handleError(errorMessage: String) {
        LogUtil.e(errorMessage)
        currentState = EMediaState.ERROR
        handleListener()

    }

    override fun pausePlay() {
        mediaPlayer?.let {
            if (currentState == EMediaState.RUNNING) {
                try {
                    it.pause()
                    currentState = EMediaState.PAUSE
                    handleListener()
                } catch (e: Exception) {
                    e.printStackTrace()
                    handleError("暂停播放异常调用")
                }
            }
        }
    }

    override fun continuePlay() {
        mediaPlayer?.let {
            when (currentState) {
                EMediaState.PAUSE -> {
                    try {
                        it.start()
                        currentState = EMediaState.RUNNING
                        handleListener()
                    } catch (e: Exception) {
                        handleError("mp.start（）异常 ${e.message}")
                        e.printStackTrace()
                    }
                }
                else ->{
                    handleError("继续播放异常调用")
                }
            }
        }
    }

    override fun stopPlay() {
        mediaPlayer?.let {
            if (currentState != EMediaState.STOP) {
                try {
                    it.stop()
                    currentState = EMediaState.STOP
                    handleListener()
                } catch (e: Exception) {
                    handleError("mp.stop（）异常 ${e.message}")
                    e.printStackTrace()
                }
            } else {
                handleError("停止播放状态异常 ${currentState.name}")
            }
        }
    }

    override fun destroyPlay() {
        mediaPlayer?.let {
            when(currentState){
                EMediaState.RUNNING,EMediaState.PAUSE,EMediaState.COMPLETE -> it.stop()
                else -> {}
            }
            it.release()
        }
        removePlayListener()

        mediaPlayer = null
    }


    override fun seekToEnable(): Boolean {
        return true
    }

    override fun getPlayPath(): String {
        return urlPath ?: ""
    }

    override fun getDuration(): Int {
        return mediaPlayer?.duration ?: -1
    }

    override fun getCurrentPosition(): Int {
        return mediaPlayer?.currentPosition ?: -1
    }

    override fun isPlaying(): Boolean {
        return mediaPlayer?.isPlaying ?: false
    }

    override fun getMediaState(): EMediaState {
        return currentState
    }

    override fun addPlayListener(listener: IMPListener) {
        this.listener = listener
    }

    override fun removePlayListener() {
        listener = null
    }

    /** 状态回调 默认状态回调是在调用MediaPlayer的线程
     * */
    private fun handleListener() {
        val temp = currentState
        synchronized(temp) {
            listener?.let {
                LogUtil.e("MPUtil发送 --》 ${Thread.currentThread()}")
                Handler(Looper.getMainLooper()).post {
                    LogUtil.e("MPUtil发送 --》 ${temp.name}")
                    it.onMpState(temp)
                }
            }
        }
    }

    override fun onPrepared(mp: MediaPlayer?) {
        LogUtil.i("mOnPreparedLister 回调")
        if (currentState == EMediaState.INITIALIZED || currentState == EMediaState.PREPARING) {
            currentState = EMediaState.PREPARED
            handlePrepared()
        } else {
            handleError("mOnPreparedLister 状态异常")
        }
    }

    override fun onError(mp: MediaPlayer?, what: Int, extra: Int): Boolean {
        handleError("mOnErrorListener 异常调用 $extra")
        return true
    }

    override fun onCompletion(mp: MediaPlayer?) {
        currentState = EMediaState.COMPLETE
        handleListener()
    }

    override fun onInfo(mp: MediaPlayer?, what: Int, extra: Int): Boolean {
        return false
    }

    override fun onSeekComplete(mp: MediaPlayer?) {
        when (currentState) {
            EMediaState.PREPARED -> handlePrepared()
            EMediaState.RUNNING,EMediaState.PAUSE -> checkSeekPlay()
            else -> {
                handleError("seekComplete state error : ${currentState.name}")
            }
        }
    }
}